mirror of
https://github.com/NVIDIA/cuda-samples.git
synced 2026-10-13 00:08:26 +08:00
CUDA 13.2 samples update (#432)
- Added Python samples for CUDA Python 1.0 release - Renamed top-level `Samples` directory to `cpp` to accommodate Python samples.
This commit is contained in:
18
cpp/5_Domain_Specific/volumeRender/.vscode/c_cpp_properties.json
vendored
Normal file
18
cpp/5_Domain_Specific/volumeRender/.vscode/c_cpp_properties.json
vendored
Normal file
@@ -0,0 +1,18 @@
|
||||
{
|
||||
"configurations": [
|
||||
{
|
||||
"name": "Linux",
|
||||
"includePath": [
|
||||
"${workspaceFolder}/**",
|
||||
"${workspaceFolder}/../../../Common"
|
||||
],
|
||||
"defines": [],
|
||||
"compilerPath": "/usr/local/cuda/bin/nvcc",
|
||||
"cStandard": "gnu17",
|
||||
"cppStandard": "gnu++14",
|
||||
"intelliSenseMode": "linux-gcc-x64",
|
||||
"configurationProvider": "ms-vscode.makefile-tools"
|
||||
}
|
||||
],
|
||||
"version": 4
|
||||
}
|
||||
7
cpp/5_Domain_Specific/volumeRender/.vscode/extensions.json
vendored
Normal file
7
cpp/5_Domain_Specific/volumeRender/.vscode/extensions.json
vendored
Normal file
@@ -0,0 +1,7 @@
|
||||
{
|
||||
"recommendations": [
|
||||
"nvidia.nsight-vscode-edition",
|
||||
"ms-vscode.cpptools",
|
||||
"ms-vscode.makefile-tools"
|
||||
]
|
||||
}
|
||||
95
cpp/5_Domain_Specific/volumeRender/CMakeLists.txt
Normal file
95
cpp/5_Domain_Specific/volumeRender/CMakeLists.txt
Normal file
@@ -0,0 +1,95 @@
|
||||
cmake_minimum_required(VERSION 3.20)
|
||||
|
||||
list(APPEND CMAKE_MODULE_PATH "${CMAKE_CURRENT_SOURCE_DIR}/../../../cmake/Modules")
|
||||
|
||||
project(volumeRender LANGUAGES C CXX CUDA)
|
||||
|
||||
find_package(CUDAToolkit REQUIRED)
|
||||
|
||||
set(CMAKE_POSITION_INDEPENDENT_CODE ON)
|
||||
|
||||
set(CMAKE_CUDA_ARCHITECTURES 75 80 86 87 89 90 100 110 120)
|
||||
set(CMAKE_CUDA_FLAGS "${CMAKE_CUDA_FLAGS} -Wno-deprecated-gpu-targets")
|
||||
if(ENABLE_CUDA_DEBUG)
|
||||
set(CMAKE_CUDA_FLAGS "${CMAKE_CUDA_FLAGS} -G") # enable cuda-gdb (may significantly affect performance on some targets)
|
||||
else()
|
||||
set(CMAKE_CUDA_FLAGS "${CMAKE_CUDA_FLAGS} -lineinfo") # add line information to all builds for debug tools (exclusive to -G option)
|
||||
endif()
|
||||
|
||||
# Include directories and libraries
|
||||
include_directories(../../../Common)
|
||||
|
||||
if(WIN32)
|
||||
set(PC_GLUT_INCLUDE_DIRS "${CMAKE_CURRENT_SOURCE_DIR}/../../../Common")
|
||||
set(PC_GLUT_LIBRARY_DIRS "${CMAKE_CURRENT_SOURCE_DIR}/../../../Common/lib/x64")
|
||||
# The GLEW library built on Windows on Arm system is named as glew32.lib/glew32.dll by default.
|
||||
if(EXISTS "${PC_GLUT_LIBRARY_DIRS}/glew32.lib")
|
||||
set(GLEW_LIB_NAME "glew32")
|
||||
else()
|
||||
set(GLEW_LIB_NAME "glew64")
|
||||
endif()
|
||||
endif()
|
||||
|
||||
find_package(OpenGL)
|
||||
find_package(GLUT)
|
||||
|
||||
# Source file
|
||||
if(${OpenGL_FOUND})
|
||||
if (${GLUT_FOUND})
|
||||
# Add target for volumeRender
|
||||
add_executable(volumeRender volumeRender_kernel.cu volumeRender.cpp)
|
||||
|
||||
target_compile_options(volumeRender PRIVATE $<$<COMPILE_LANGUAGE:CUDA>:--extended-lambda>)
|
||||
|
||||
target_compile_features(volumeRender PRIVATE cxx_std_17 cuda_std_17)
|
||||
|
||||
set_target_properties(volumeRender PROPERTIES CUDA_SEPARABLE_COMPILATION ON)
|
||||
|
||||
target_include_directories(volumeRender PUBLIC
|
||||
${OPENGL_INCLUDE_DIR}
|
||||
${CUDAToolkit_INCLUDE_DIRS}
|
||||
${GLUT_INCLUDE_DIRS}
|
||||
)
|
||||
|
||||
target_link_libraries(volumeRender
|
||||
${OPENGL_LIBRARIES}
|
||||
${GLUT_LIBRARIES}
|
||||
)
|
||||
|
||||
add_custom_command(TARGET volumeRender POST_BUILD
|
||||
COMMAND ${CMAKE_COMMAND} -E copy_directory
|
||||
${CMAKE_CURRENT_SOURCE_DIR}/data
|
||||
${CMAKE_CURRENT_BINARY_DIR}/data
|
||||
)
|
||||
|
||||
if(WIN32)
|
||||
target_link_libraries(volumeRender
|
||||
${PC_GLUT_LIBRARY_DIRS}/freeglut.lib
|
||||
${PC_GLUT_LIBRARY_DIRS}/${GLEW_LIB_NAME}.lib
|
||||
)
|
||||
|
||||
add_custom_command(TARGET volumeRender
|
||||
POST_BUILD
|
||||
COMMAND ${CMAKE_COMMAND} -E copy
|
||||
${CMAKE_CURRENT_SOURCE_DIR}/../../../bin/win64/$<CONFIGURATION>/freeglut.dll
|
||||
${CMAKE_CURRENT_BINARY_DIR}/$<CONFIGURATION>
|
||||
)
|
||||
|
||||
add_custom_command(TARGET volumeRender
|
||||
POST_BUILD
|
||||
COMMAND ${CMAKE_COMMAND} -E copy
|
||||
${CMAKE_CURRENT_SOURCE_DIR}/../../../bin/win64/$<CONFIGURATION>/${GLEW_LIB_NAME}.dll
|
||||
${CMAKE_CURRENT_BINARY_DIR}/$<CONFIGURATION>
|
||||
)
|
||||
endif()
|
||||
|
||||
else()
|
||||
message(STATUS "GLUT not found - will not build sample 'volumeRender'")
|
||||
endif()
|
||||
else()
|
||||
message(STATUS "OpenGL not found - will not build sample 'volumeRender'")
|
||||
endif()
|
||||
|
||||
# Include installation configuration
|
||||
include(${CMAKE_CURRENT_SOURCE_DIR}/../../../cmake/InstallSamples.cmake)
|
||||
setup_samples_install()
|
||||
36
cpp/5_Domain_Specific/volumeRender/README.md
Normal file
36
cpp/5_Domain_Specific/volumeRender/README.md
Normal file
@@ -0,0 +1,36 @@
|
||||
# volumeRender - Volume Rendering with 3D Textures
|
||||
|
||||
## Description
|
||||
|
||||
This sample demonstrates basic volume rendering using 3D Textures.
|
||||
|
||||
## Key Concepts
|
||||
|
||||
Graphics Interop, Image Processing, 3D Textures
|
||||
|
||||
## Supported SM Architectures
|
||||
|
||||
[SM 5.0 ](https://developer.nvidia.com/cuda-gpus) [SM 5.2 ](https://developer.nvidia.com/cuda-gpus) [SM 5.3 ](https://developer.nvidia.com/cuda-gpus) [SM 6.0 ](https://developer.nvidia.com/cuda-gpus) [SM 6.1 ](https://developer.nvidia.com/cuda-gpus) [SM 7.0 ](https://developer.nvidia.com/cuda-gpus) [SM 7.2 ](https://developer.nvidia.com/cuda-gpus) [SM 7.5 ](https://developer.nvidia.com/cuda-gpus) [SM 8.0 ](https://developer.nvidia.com/cuda-gpus) [SM 8.6 ](https://developer.nvidia.com/cuda-gpus) [SM 8.7 ](https://developer.nvidia.com/cuda-gpus) [SM 8.9 ](https://developer.nvidia.com/cuda-gpus) [SM 9.0 ](https://developer.nvidia.com/cuda-gpus)
|
||||
|
||||
## Supported OSes
|
||||
|
||||
Linux, Windows
|
||||
|
||||
## Supported CPU Architecture
|
||||
|
||||
x86_64, armv7l
|
||||
|
||||
## CUDA APIs involved
|
||||
|
||||
### [CUDA Runtime API](http://docs.nvidia.com/cuda/cuda-runtime-api/index.html)
|
||||
cudaProfilerStop, cudaGraphicsUnmapResources, cudaMemcpy, cudaMallocArray, cudaFreeArray, cudaFree, cudaPitchedPtr, cudaGraphicsResourceGetMappedPointer, cudaGraphicsMapResources, cudaDestroyTextureObject, cudaExtent, cudaDeviceSynchronize, cudaCreateTextureObject, cudaMemset, cudaMemcpyToSymbol, cudaGraphicsUnregisterResource, cudaMalloc, cudaGraphicsGLRegisterBuffer
|
||||
|
||||
## Dependencies needed to build/run
|
||||
[X11](../../../README.md#x11), [OpenGL](../../../README.md#opengl), [Freeglut](../../../README.md#freeglut), [GLEW](../../../README.md#glew)
|
||||
|
||||
## Prerequisites
|
||||
|
||||
Download and install the [CUDA Toolkit](https://developer.nvidia.com/cuda-downloads) for your corresponding platform.
|
||||
Make sure the dependencies mentioned in [Dependencies]() section above are installed.
|
||||
|
||||
## References (for more details)
|
||||
BIN
cpp/5_Domain_Specific/volumeRender/data/Bucky.raw
Normal file
BIN
cpp/5_Domain_Specific/volumeRender/data/Bucky.raw
Normal file
Binary file not shown.
BIN
cpp/5_Domain_Specific/volumeRender/data/ref_volume.ppm
Normal file
BIN
cpp/5_Domain_Specific/volumeRender/data/ref_volume.ppm
Normal file
Binary file not shown.
BIN
cpp/5_Domain_Specific/volumeRender/doc/sshot_lg.JPG
Normal file
BIN
cpp/5_Domain_Specific/volumeRender/doc/sshot_lg.JPG
Normal file
Binary file not shown.
|
After Width: | Height: | Size: 101 KiB |
BIN
cpp/5_Domain_Specific/volumeRender/doc/sshot_md.jpg
Normal file
BIN
cpp/5_Domain_Specific/volumeRender/doc/sshot_md.jpg
Normal file
Binary file not shown.
|
After Width: | Height: | Size: 36 KiB |
BIN
cpp/5_Domain_Specific/volumeRender/doc/sshot_sm.JPG
Normal file
BIN
cpp/5_Domain_Specific/volumeRender/doc/sshot_sm.JPG
Normal file
Binary file not shown.
|
After Width: | Height: | Size: 18 KiB |
BIN
cpp/5_Domain_Specific/volumeRender/volume.ppm
Normal file
BIN
cpp/5_Domain_Specific/volumeRender/volume.ppm
Normal file
Binary file not shown.
648
cpp/5_Domain_Specific/volumeRender/volumeRender.cpp
Normal file
648
cpp/5_Domain_Specific/volumeRender/volumeRender.cpp
Normal file
@@ -0,0 +1,648 @@
|
||||
/* Copyright (c) 2022, NVIDIA CORPORATION. All rights reserved.
|
||||
*
|
||||
* Redistribution and use in source and binary forms, with or without
|
||||
* modification, are permitted provided that the following conditions
|
||||
* are met:
|
||||
* * Redistributions of source code must retain the above copyright
|
||||
* notice, this list of conditions and the following disclaimer.
|
||||
* * Redistributions in binary form must reproduce the above copyright
|
||||
* notice, this list of conditions and the following disclaimer in the
|
||||
* documentation and/or other materials provided with the distribution.
|
||||
* * Neither the name of NVIDIA CORPORATION nor the names of its
|
||||
* contributors may be used to endorse or promote products derived
|
||||
* from this software without specific prior written permission.
|
||||
*
|
||||
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ``AS IS'' AND ANY
|
||||
* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
|
||||
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
|
||||
* PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
|
||||
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
|
||||
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
|
||||
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
|
||||
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
|
||||
* OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
|
||||
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
|
||||
* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
|
||||
*/
|
||||
|
||||
/*
|
||||
Volume rendering sample
|
||||
|
||||
This sample loads a 3D volume from disk and displays it using
|
||||
ray marching and 3D textures.
|
||||
|
||||
Note - this is intended to be an example of using 3D textures
|
||||
in CUDA, not an optimized volume renderer.
|
||||
|
||||
Changes
|
||||
sgg 22/3/2010
|
||||
- updated to use texture for display instead of glDrawPixels.
|
||||
- changed to render from front-to-back rather than back-to-front.
|
||||
*/
|
||||
|
||||
// OpenGL Graphics includes
|
||||
#include <helper_gl.h>
|
||||
#if defined(__APPLE__) || defined(MACOSX)
|
||||
#pragma clang diagnostic ignored "-Wdeprecated-declarations"
|
||||
#include <GLUT/glut.h>
|
||||
#ifndef glutCloseFunc
|
||||
#define glutCloseFunc glutWMCloseFunc
|
||||
#endif
|
||||
#else
|
||||
#include <GL/freeglut.h>
|
||||
#endif
|
||||
|
||||
// CUDA Runtime, Interop, and includes
|
||||
#include <cuda_gl_interop.h>
|
||||
#include <cuda_profiler_api.h>
|
||||
#include <cuda_runtime.h>
|
||||
#include <driver_functions.h>
|
||||
#include <vector_functions.h>
|
||||
#include <vector_types.h>
|
||||
|
||||
// CUDA utilities
|
||||
#include <helper_cuda.h>
|
||||
|
||||
// Helper functions
|
||||
#include <helper_cuda.h>
|
||||
#include <helper_functions.h>
|
||||
#include <helper_timer.h>
|
||||
|
||||
typedef unsigned int uint;
|
||||
typedef unsigned char uchar;
|
||||
|
||||
#define MAX_EPSILON_ERROR 5.00f
|
||||
#define THRESHOLD 0.30f
|
||||
|
||||
// Define the files that are to be save and the reference images for validation
|
||||
const char *sOriginal[] = {"volume.ppm", NULL};
|
||||
|
||||
const char *sReference[] = {"ref_volume.ppm", NULL};
|
||||
|
||||
const char *sSDKsample = "CUDA 3D Volume Render";
|
||||
|
||||
const char *volumeFilename = "Bucky.raw";
|
||||
cudaExtent volumeSize = make_cudaExtent(32, 32, 32);
|
||||
typedef unsigned char VolumeType;
|
||||
|
||||
// char *volumeFilename = "mrt16_angio.raw";
|
||||
// cudaExtent volumeSize = make_cudaExtent(416, 512, 112);
|
||||
// typedef unsigned short VolumeType;
|
||||
|
||||
uint width = 512, height = 512;
|
||||
dim3 blockSize(16, 16);
|
||||
dim3 gridSize;
|
||||
|
||||
float3 viewRotation;
|
||||
float3 viewTranslation = make_float3(0.0, 0.0, -4.0f);
|
||||
float invViewMatrix[12];
|
||||
|
||||
float density = 0.05f;
|
||||
float brightness = 1.0f;
|
||||
float transferOffset = 0.0f;
|
||||
float transferScale = 1.0f;
|
||||
bool linearFiltering = true;
|
||||
|
||||
GLuint pbo = 0; // OpenGL pixel buffer object
|
||||
GLuint tex = 0; // OpenGL texture object
|
||||
struct cudaGraphicsResource *cuda_pbo_resource; // CUDA Graphics Resource (to transfer PBO)
|
||||
|
||||
StopWatchInterface *timer = 0;
|
||||
|
||||
// Auto-Verification Code
|
||||
const int frameCheckNumber = 2;
|
||||
int fpsCount = 0; // FPS count for averaging
|
||||
int fpsLimit = 1; // FPS limit for sampling
|
||||
int g_Index = 0;
|
||||
unsigned int frameCount = 0;
|
||||
|
||||
int *pArgc;
|
||||
char **pArgv;
|
||||
|
||||
#ifndef MAX
|
||||
#define MAX(a, b) ((a > b) ? a : b)
|
||||
#endif
|
||||
|
||||
extern "C" void setTextureFilterMode(bool bLinearFilter);
|
||||
extern "C" void initCuda(void *h_volume, cudaExtent volumeSize);
|
||||
extern "C" void freeCudaBuffers();
|
||||
extern "C" void render_kernel(dim3 gridSize,
|
||||
dim3 blockSize,
|
||||
uint *d_output,
|
||||
uint imageW,
|
||||
uint imageH,
|
||||
float density,
|
||||
float brightness,
|
||||
float transferOffset,
|
||||
float transferScale);
|
||||
extern "C" void copyInvViewMatrix(float *invViewMatrix, size_t sizeofMatrix);
|
||||
|
||||
void initPixelBuffer();
|
||||
|
||||
void computeFPS()
|
||||
{
|
||||
frameCount++;
|
||||
fpsCount++;
|
||||
|
||||
if (fpsCount == fpsLimit) {
|
||||
char fps[256];
|
||||
float ifps = 1.f / (sdkGetAverageTimerValue(&timer) / 1000.f);
|
||||
sprintf(fps, "Volume Render: %3.1f fps", ifps);
|
||||
|
||||
glutSetWindowTitle(fps);
|
||||
fpsCount = 0;
|
||||
|
||||
fpsLimit = (int)MAX(1.f, ifps);
|
||||
sdkResetTimer(&timer);
|
||||
}
|
||||
}
|
||||
|
||||
// render image using CUDA
|
||||
void render()
|
||||
{
|
||||
copyInvViewMatrix(invViewMatrix, sizeof(float4) * 3);
|
||||
|
||||
// map PBO to get CUDA device pointer
|
||||
uint *d_output;
|
||||
// map PBO to get CUDA device pointer
|
||||
checkCudaErrors(cudaGraphicsMapResources(1, &cuda_pbo_resource, 0));
|
||||
size_t num_bytes;
|
||||
checkCudaErrors(cudaGraphicsResourceGetMappedPointer((void **)&d_output, &num_bytes, cuda_pbo_resource));
|
||||
// printf("CUDA mapped PBO: May access %ld bytes\n", num_bytes);
|
||||
|
||||
// clear image
|
||||
checkCudaErrors(cudaMemset(d_output, 0, width * height * 4));
|
||||
|
||||
// call CUDA kernel, writing results to PBO
|
||||
render_kernel(gridSize, blockSize, d_output, width, height, density, brightness, transferOffset, transferScale);
|
||||
|
||||
getLastCudaError("kernel failed");
|
||||
|
||||
checkCudaErrors(cudaGraphicsUnmapResources(1, &cuda_pbo_resource, 0));
|
||||
}
|
||||
|
||||
// display results using OpenGL (called by GLUT)
|
||||
void display()
|
||||
{
|
||||
sdkStartTimer(&timer);
|
||||
|
||||
// use OpenGL to build view matrix
|
||||
GLfloat modelView[16];
|
||||
glMatrixMode(GL_MODELVIEW);
|
||||
glPushMatrix();
|
||||
glLoadIdentity();
|
||||
glRotatef(-viewRotation.x, 1.0, 0.0, 0.0);
|
||||
glRotatef(-viewRotation.y, 0.0, 1.0, 0.0);
|
||||
glTranslatef(-viewTranslation.x, -viewTranslation.y, -viewTranslation.z);
|
||||
glGetFloatv(GL_MODELVIEW_MATRIX, modelView);
|
||||
glPopMatrix();
|
||||
|
||||
invViewMatrix[0] = modelView[0];
|
||||
invViewMatrix[1] = modelView[4];
|
||||
invViewMatrix[2] = modelView[8];
|
||||
invViewMatrix[3] = modelView[12];
|
||||
invViewMatrix[4] = modelView[1];
|
||||
invViewMatrix[5] = modelView[5];
|
||||
invViewMatrix[6] = modelView[9];
|
||||
invViewMatrix[7] = modelView[13];
|
||||
invViewMatrix[8] = modelView[2];
|
||||
invViewMatrix[9] = modelView[6];
|
||||
invViewMatrix[10] = modelView[10];
|
||||
invViewMatrix[11] = modelView[14];
|
||||
|
||||
render();
|
||||
|
||||
// display results
|
||||
glClear(GL_COLOR_BUFFER_BIT);
|
||||
|
||||
// draw image from PBO
|
||||
glDisable(GL_DEPTH_TEST);
|
||||
|
||||
glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
|
||||
#if 0
|
||||
// draw using glDrawPixels (slower)
|
||||
glRasterPos2i(0, 0);
|
||||
glBindBuffer(GL_PIXEL_UNPACK_BUFFER_ARB, pbo);
|
||||
glDrawPixels(width, height, GL_RGBA, GL_UNSIGNED_BYTE, 0);
|
||||
glBindBuffer(GL_PIXEL_UNPACK_BUFFER_ARB, 0);
|
||||
#else
|
||||
// draw using texture
|
||||
|
||||
// copy from pbo to texture
|
||||
glBindBuffer(GL_PIXEL_UNPACK_BUFFER_ARB, pbo);
|
||||
glBindTexture(GL_TEXTURE_2D, tex);
|
||||
glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, width, height, GL_RGBA, GL_UNSIGNED_BYTE, 0);
|
||||
glBindBuffer(GL_PIXEL_UNPACK_BUFFER_ARB, 0);
|
||||
|
||||
// draw textured quad
|
||||
glEnable(GL_TEXTURE_2D);
|
||||
glBegin(GL_QUADS);
|
||||
glTexCoord2f(0, 0);
|
||||
glVertex2f(0, 0);
|
||||
glTexCoord2f(1, 0);
|
||||
glVertex2f(1, 0);
|
||||
glTexCoord2f(1, 1);
|
||||
glVertex2f(1, 1);
|
||||
glTexCoord2f(0, 1);
|
||||
glVertex2f(0, 1);
|
||||
glEnd();
|
||||
|
||||
glDisable(GL_TEXTURE_2D);
|
||||
glBindTexture(GL_TEXTURE_2D, 0);
|
||||
#endif
|
||||
|
||||
glutSwapBuffers();
|
||||
glutReportErrors();
|
||||
|
||||
sdkStopTimer(&timer);
|
||||
|
||||
computeFPS();
|
||||
}
|
||||
|
||||
void idle() { glutPostRedisplay(); }
|
||||
|
||||
void keyboard(unsigned char key, int x, int y)
|
||||
{
|
||||
switch (key) {
|
||||
case 27:
|
||||
#if defined(__APPLE__) || defined(MACOSX)
|
||||
exit(EXIT_SUCCESS);
|
||||
#else
|
||||
glutDestroyWindow(glutGetWindow());
|
||||
return;
|
||||
#endif
|
||||
break;
|
||||
|
||||
case 'f':
|
||||
linearFiltering = !linearFiltering;
|
||||
setTextureFilterMode(linearFiltering);
|
||||
break;
|
||||
|
||||
case '+':
|
||||
density += 0.01f;
|
||||
break;
|
||||
|
||||
case '-':
|
||||
density -= 0.01f;
|
||||
break;
|
||||
|
||||
case ']':
|
||||
brightness += 0.1f;
|
||||
break;
|
||||
|
||||
case '[':
|
||||
brightness -= 0.1f;
|
||||
break;
|
||||
|
||||
case ';':
|
||||
transferOffset += 0.01f;
|
||||
break;
|
||||
|
||||
case '\'':
|
||||
transferOffset -= 0.01f;
|
||||
break;
|
||||
|
||||
case '.':
|
||||
transferScale += 0.01f;
|
||||
break;
|
||||
|
||||
case ',':
|
||||
transferScale -= 0.01f;
|
||||
break;
|
||||
|
||||
default:
|
||||
break;
|
||||
}
|
||||
|
||||
printf("density = %.2f, brightness = %.2f, transferOffset = %.2f, transferScale "
|
||||
"= %.2f\n",
|
||||
density,
|
||||
brightness,
|
||||
transferOffset,
|
||||
transferScale);
|
||||
glutPostRedisplay();
|
||||
}
|
||||
|
||||
int ox, oy;
|
||||
int buttonState = 0;
|
||||
|
||||
void mouse(int button, int state, int x, int y)
|
||||
{
|
||||
if (state == GLUT_DOWN) {
|
||||
buttonState |= 1 << button;
|
||||
}
|
||||
else if (state == GLUT_UP) {
|
||||
buttonState = 0;
|
||||
}
|
||||
|
||||
ox = x;
|
||||
oy = y;
|
||||
glutPostRedisplay();
|
||||
}
|
||||
|
||||
void motion(int x, int y)
|
||||
{
|
||||
float dx, dy;
|
||||
dx = (float)(x - ox);
|
||||
dy = (float)(y - oy);
|
||||
|
||||
if (buttonState == 4) {
|
||||
// right = zoom
|
||||
viewTranslation.z += dy / 100.0f;
|
||||
}
|
||||
else if (buttonState == 2) {
|
||||
// middle = translate
|
||||
viewTranslation.x += dx / 100.0f;
|
||||
viewTranslation.y -= dy / 100.0f;
|
||||
}
|
||||
else if (buttonState == 1) {
|
||||
// left = rotate
|
||||
viewRotation.x += dy / 5.0f;
|
||||
viewRotation.y += dx / 5.0f;
|
||||
}
|
||||
|
||||
ox = x;
|
||||
oy = y;
|
||||
glutPostRedisplay();
|
||||
}
|
||||
|
||||
int iDivUp(int a, int b) { return (a % b != 0) ? (a / b + 1) : (a / b); }
|
||||
|
||||
void reshape(int w, int h)
|
||||
{
|
||||
width = w;
|
||||
height = h;
|
||||
initPixelBuffer();
|
||||
|
||||
// calculate new grid size
|
||||
gridSize = dim3(iDivUp(width, blockSize.x), iDivUp(height, blockSize.y));
|
||||
|
||||
glViewport(0, 0, w, h);
|
||||
|
||||
glMatrixMode(GL_MODELVIEW);
|
||||
glLoadIdentity();
|
||||
|
||||
glMatrixMode(GL_PROJECTION);
|
||||
glLoadIdentity();
|
||||
glOrtho(0.0, 1.0, 0.0, 1.0, 0.0, 1.0);
|
||||
}
|
||||
|
||||
void cleanup()
|
||||
{
|
||||
sdkDeleteTimer(&timer);
|
||||
|
||||
freeCudaBuffers();
|
||||
|
||||
if (pbo) {
|
||||
cudaGraphicsUnregisterResource(cuda_pbo_resource);
|
||||
glDeleteBuffers(1, &pbo);
|
||||
glDeleteTextures(1, &tex);
|
||||
}
|
||||
// Calling cudaProfilerStop causes all profile data to be
|
||||
// flushed before the application exits
|
||||
checkCudaErrors(cudaProfilerStop());
|
||||
}
|
||||
|
||||
void initGL(int *argc, char **argv)
|
||||
{
|
||||
// initialize GLUT callback functions
|
||||
glutInit(argc, argv);
|
||||
glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE);
|
||||
glutInitWindowSize(width, height);
|
||||
glutCreateWindow("CUDA volume rendering");
|
||||
|
||||
if (!isGLVersionSupported(2, 0) || !areGLExtensionsSupported("GL_ARB_pixel_buffer_object")) {
|
||||
printf("Required OpenGL extensions are missing.");
|
||||
exit(EXIT_SUCCESS);
|
||||
}
|
||||
}
|
||||
|
||||
void initPixelBuffer()
|
||||
{
|
||||
if (pbo) {
|
||||
// unregister this buffer object from CUDA C
|
||||
checkCudaErrors(cudaGraphicsUnregisterResource(cuda_pbo_resource));
|
||||
|
||||
// delete old buffer
|
||||
glDeleteBuffers(1, &pbo);
|
||||
glDeleteTextures(1, &tex);
|
||||
}
|
||||
|
||||
// create pixel buffer object for display
|
||||
glGenBuffers(1, &pbo);
|
||||
glBindBuffer(GL_PIXEL_UNPACK_BUFFER_ARB, pbo);
|
||||
glBufferData(GL_PIXEL_UNPACK_BUFFER_ARB, width * height * sizeof(GLubyte) * 4, 0, GL_STREAM_DRAW_ARB);
|
||||
glBindBuffer(GL_PIXEL_UNPACK_BUFFER_ARB, 0);
|
||||
|
||||
// register this buffer object with CUDA
|
||||
checkCudaErrors(cudaGraphicsGLRegisterBuffer(&cuda_pbo_resource, pbo, cudaGraphicsMapFlagsWriteDiscard));
|
||||
|
||||
// create texture for display
|
||||
glGenTextures(1, &tex);
|
||||
glBindTexture(GL_TEXTURE_2D, tex);
|
||||
glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, NULL);
|
||||
glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
|
||||
glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
|
||||
glBindTexture(GL_TEXTURE_2D, 0);
|
||||
}
|
||||
|
||||
// Load raw data from disk
|
||||
void *loadRawFile(char *filename, size_t size)
|
||||
{
|
||||
FILE *fp = fopen(filename, "rb");
|
||||
|
||||
if (!fp) {
|
||||
fprintf(stderr, "Error opening file '%s'\n", filename);
|
||||
return 0;
|
||||
}
|
||||
|
||||
void *data = malloc(size);
|
||||
size_t read = fread(data, 1, size, fp);
|
||||
fclose(fp);
|
||||
|
||||
#if defined(_MSC_VER_)
|
||||
printf("Read '%s', %Iu bytes\n", filename, read);
|
||||
#else
|
||||
printf("Read '%s', %zu bytes\n", filename, read);
|
||||
#endif
|
||||
|
||||
return data;
|
||||
}
|
||||
|
||||
void runSingleTest(const char *ref_file, const char *exec_path)
|
||||
{
|
||||
bool bTestResult = true;
|
||||
|
||||
uint *d_output;
|
||||
checkCudaErrors(cudaMalloc((void **)&d_output, width * height * sizeof(uint)));
|
||||
checkCudaErrors(cudaMemset(d_output, 0, width * height * sizeof(uint)));
|
||||
|
||||
float modelView[16] = {
|
||||
1.0f, 0.0f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f, 0.0f, 4.0f, 1.0f};
|
||||
|
||||
invViewMatrix[0] = modelView[0];
|
||||
invViewMatrix[1] = modelView[4];
|
||||
invViewMatrix[2] = modelView[8];
|
||||
invViewMatrix[3] = modelView[12];
|
||||
invViewMatrix[4] = modelView[1];
|
||||
invViewMatrix[5] = modelView[5];
|
||||
invViewMatrix[6] = modelView[9];
|
||||
invViewMatrix[7] = modelView[13];
|
||||
invViewMatrix[8] = modelView[2];
|
||||
invViewMatrix[9] = modelView[6];
|
||||
invViewMatrix[10] = modelView[10];
|
||||
invViewMatrix[11] = modelView[14];
|
||||
|
||||
// call CUDA kernel, writing results to PBO
|
||||
copyInvViewMatrix(invViewMatrix, sizeof(float4) * 3);
|
||||
|
||||
// Start timer 0 and process n loops on the GPU
|
||||
int nIter = 10;
|
||||
|
||||
for (int i = -1; i < nIter; i++) {
|
||||
if (i == 0) {
|
||||
cudaDeviceSynchronize();
|
||||
sdkStartTimer(&timer);
|
||||
}
|
||||
|
||||
render_kernel(gridSize, blockSize, d_output, width, height, density, brightness, transferOffset, transferScale);
|
||||
}
|
||||
|
||||
cudaDeviceSynchronize();
|
||||
sdkStopTimer(&timer);
|
||||
// Get elapsed time and throughput, then log to sample and master logs
|
||||
double dAvgTime = sdkGetTimerValue(&timer) / (nIter * 1000.0);
|
||||
printf("volumeRender, Throughput = %.4f MTexels/s, Time = %.5f s, Size = %u "
|
||||
"Texels, NumDevsUsed = %u, Workgroup = %u\n",
|
||||
(1.0e-6 * width * height) / dAvgTime,
|
||||
dAvgTime,
|
||||
(width * height),
|
||||
1,
|
||||
blockSize.x * blockSize.y);
|
||||
|
||||
getLastCudaError("Error: render_kernel() execution FAILED");
|
||||
checkCudaErrors(cudaDeviceSynchronize());
|
||||
|
||||
unsigned char *h_output = (unsigned char *)malloc(width * height * 4);
|
||||
checkCudaErrors(cudaMemcpy(h_output, d_output, width * height * 4, cudaMemcpyDeviceToHost));
|
||||
|
||||
sdkSavePPM4ub("volume.ppm", h_output, width, height);
|
||||
bTestResult = sdkComparePPM("volume.ppm", sdkFindFilePath(ref_file, exec_path), MAX_EPSILON_ERROR, THRESHOLD, true);
|
||||
|
||||
cudaFree(d_output);
|
||||
free(h_output);
|
||||
cleanup();
|
||||
|
||||
exit(bTestResult ? EXIT_SUCCESS : EXIT_FAILURE);
|
||||
}
|
||||
|
||||
////////////////////////////////////////////////////////////////////////////////
|
||||
// Program main
|
||||
////////////////////////////////////////////////////////////////////////////////
|
||||
int main(int argc, char **argv)
|
||||
{
|
||||
pArgc = &argc;
|
||||
pArgv = argv;
|
||||
|
||||
char *ref_file = NULL;
|
||||
|
||||
#if defined(__linux__)
|
||||
setenv("DISPLAY", ":0", 0);
|
||||
#endif
|
||||
|
||||
// start logs
|
||||
printf("%s Starting...\n\n", sSDKsample);
|
||||
|
||||
if (checkCmdLineFlag(argc, (const char **)argv, "file")) {
|
||||
getCmdLineArgumentString(argc, (const char **)argv, "file", &ref_file);
|
||||
fpsLimit = frameCheckNumber;
|
||||
}
|
||||
|
||||
if (ref_file) {
|
||||
findCudaDevice(argc, (const char **)argv);
|
||||
}
|
||||
else {
|
||||
// First initialize OpenGL context, so we can properly set the GL for CUDA.
|
||||
// This is necessary in order to achieve optimal performance with
|
||||
// OpenGL/CUDA interop.
|
||||
initGL(&argc, argv);
|
||||
|
||||
findCudaDevice(argc, (const char **)argv);
|
||||
}
|
||||
|
||||
// parse arguments
|
||||
char *filename;
|
||||
|
||||
if (getCmdLineArgumentString(argc, (const char **)argv, "volume", &filename)) {
|
||||
volumeFilename = filename;
|
||||
}
|
||||
|
||||
int n;
|
||||
|
||||
if (checkCmdLineFlag(argc, (const char **)argv, "size")) {
|
||||
n = getCmdLineArgumentInt(argc, (const char **)argv, "size");
|
||||
volumeSize.width = volumeSize.height = volumeSize.depth = n;
|
||||
}
|
||||
|
||||
if (checkCmdLineFlag(argc, (const char **)argv, "xsize")) {
|
||||
n = getCmdLineArgumentInt(argc, (const char **)argv, "xsize");
|
||||
volumeSize.width = n;
|
||||
}
|
||||
|
||||
if (checkCmdLineFlag(argc, (const char **)argv, "ysize")) {
|
||||
n = getCmdLineArgumentInt(argc, (const char **)argv, "ysize");
|
||||
volumeSize.height = n;
|
||||
}
|
||||
|
||||
if (checkCmdLineFlag(argc, (const char **)argv, "zsize")) {
|
||||
n = getCmdLineArgumentInt(argc, (const char **)argv, "zsize");
|
||||
volumeSize.depth = n;
|
||||
}
|
||||
|
||||
// load volume data
|
||||
char *path = sdkFindFilePath(volumeFilename, argv[0]);
|
||||
|
||||
if (path == 0) {
|
||||
printf("Error finding file '%s'\n", volumeFilename);
|
||||
exit(EXIT_FAILURE);
|
||||
}
|
||||
|
||||
size_t size = volumeSize.width * volumeSize.height * volumeSize.depth * sizeof(VolumeType);
|
||||
void *h_volume = loadRawFile(path, size);
|
||||
|
||||
initCuda(h_volume, volumeSize);
|
||||
free(h_volume);
|
||||
|
||||
sdkCreateTimer(&timer);
|
||||
|
||||
printf("Press '+' and '-' to change density (0.01 increments)\n"
|
||||
" ']' and '[' to change brightness\n"
|
||||
" ';' and ''' to modify transfer function offset\n"
|
||||
" '.' and ',' to modify transfer function scale\n\n");
|
||||
|
||||
// calculate new grid size
|
||||
gridSize = dim3(iDivUp(width, blockSize.x), iDivUp(height, blockSize.y));
|
||||
|
||||
if (ref_file) {
|
||||
runSingleTest(ref_file, argv[0]);
|
||||
}
|
||||
else {
|
||||
// This is the normal rendering path for VolumeRender
|
||||
glutDisplayFunc(display);
|
||||
glutKeyboardFunc(keyboard);
|
||||
glutMouseFunc(mouse);
|
||||
glutMotionFunc(motion);
|
||||
glutReshapeFunc(reshape);
|
||||
glutIdleFunc(idle);
|
||||
|
||||
initPixelBuffer();
|
||||
|
||||
#if defined(__APPLE__) || defined(MACOSX)
|
||||
atexit(cleanup);
|
||||
#else
|
||||
glutCloseFunc(cleanup);
|
||||
#endif
|
||||
|
||||
glutMainLoop();
|
||||
}
|
||||
}
|
||||
369
cpp/5_Domain_Specific/volumeRender/volumeRender_kernel.cu
Normal file
369
cpp/5_Domain_Specific/volumeRender/volumeRender_kernel.cu
Normal file
@@ -0,0 +1,369 @@
|
||||
/* Copyright (c) 2022, NVIDIA CORPORATION. All rights reserved.
|
||||
*
|
||||
* Redistribution and use in source and binary forms, with or without
|
||||
* modification, are permitted provided that the following conditions
|
||||
* are met:
|
||||
* * Redistributions of source code must retain the above copyright
|
||||
* notice, this list of conditions and the following disclaimer.
|
||||
* * Redistributions in binary form must reproduce the above copyright
|
||||
* notice, this list of conditions and the following disclaimer in the
|
||||
* documentation and/or other materials provided with the distribution.
|
||||
* * Neither the name of NVIDIA CORPORATION nor the names of its
|
||||
* contributors may be used to endorse or promote products derived
|
||||
* from this software without specific prior written permission.
|
||||
*
|
||||
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ``AS IS'' AND ANY
|
||||
* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
|
||||
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
|
||||
* PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
|
||||
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
|
||||
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
|
||||
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
|
||||
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
|
||||
* OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
|
||||
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
|
||||
* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
|
||||
*/
|
||||
|
||||
// Simple 3D volume renderer
|
||||
|
||||
#ifndef _VOLUMERENDER_KERNEL_CU_
|
||||
#define _VOLUMERENDER_KERNEL_CU_
|
||||
|
||||
#include <helper_cuda.h>
|
||||
#include <helper_math.h>
|
||||
|
||||
typedef unsigned int uint;
|
||||
typedef unsigned char uchar;
|
||||
|
||||
cudaArray *d_volumeArray = 0;
|
||||
cudaArray *d_transferFuncArray;
|
||||
|
||||
typedef unsigned char VolumeType;
|
||||
// typedef unsigned short VolumeType;
|
||||
|
||||
cudaTextureObject_t texObject; // For 3D texture
|
||||
cudaTextureObject_t transferTex; // For 1D transfer function texture
|
||||
|
||||
typedef struct
|
||||
{
|
||||
float4 m[3];
|
||||
} float3x4;
|
||||
|
||||
__constant__ float3x4 c_invViewMatrix; // inverse view matrix
|
||||
|
||||
struct Ray
|
||||
{
|
||||
float3 o; // origin
|
||||
float3 d; // direction
|
||||
};
|
||||
|
||||
// intersect ray with a box
|
||||
// http://www.siggraph.org/education/materials/HyperGraph/raytrace/rtinter3.htm
|
||||
|
||||
__device__ int intersectBox(Ray r, float3 boxmin, float3 boxmax, float *tnear, float *tfar)
|
||||
{
|
||||
// compute intersection of ray with all six bbox planes
|
||||
float3 invR = make_float3(1.0f) / r.d;
|
||||
float3 tbot = invR * (boxmin - r.o);
|
||||
float3 ttop = invR * (boxmax - r.o);
|
||||
|
||||
// re-order intersections to find smallest and largest on each axis
|
||||
float3 tmin = fminf(ttop, tbot);
|
||||
float3 tmax = fmaxf(ttop, tbot);
|
||||
|
||||
// find the largest tmin and the smallest tmax
|
||||
float largest_tmin = fmaxf(fmaxf(tmin.x, tmin.y), fmaxf(tmin.x, tmin.z));
|
||||
float smallest_tmax = fminf(fminf(tmax.x, tmax.y), fminf(tmax.x, tmax.z));
|
||||
|
||||
*tnear = largest_tmin;
|
||||
*tfar = smallest_tmax;
|
||||
|
||||
return smallest_tmax > largest_tmin;
|
||||
}
|
||||
|
||||
// transform vector by matrix (no translation)
|
||||
__device__ float3 mul(const float3x4 &M, const float3 &v)
|
||||
{
|
||||
float3 r;
|
||||
r.x = dot(v, make_float3(M.m[0]));
|
||||
r.y = dot(v, make_float3(M.m[1]));
|
||||
r.z = dot(v, make_float3(M.m[2]));
|
||||
return r;
|
||||
}
|
||||
|
||||
// transform vector by matrix with translation
|
||||
__device__ float4 mul(const float3x4 &M, const float4 &v)
|
||||
{
|
||||
float4 r;
|
||||
r.x = dot(v, M.m[0]);
|
||||
r.y = dot(v, M.m[1]);
|
||||
r.z = dot(v, M.m[2]);
|
||||
r.w = 1.0f;
|
||||
return r;
|
||||
}
|
||||
|
||||
__device__ uint rgbaFloatToInt(float4 rgba)
|
||||
{
|
||||
rgba.x = __saturatef(rgba.x); // clamp to [0.0, 1.0]
|
||||
rgba.y = __saturatef(rgba.y);
|
||||
rgba.z = __saturatef(rgba.z);
|
||||
rgba.w = __saturatef(rgba.w);
|
||||
return (uint(rgba.w * 255) << 24) | (uint(rgba.z * 255) << 16) | (uint(rgba.y * 255) << 8) | uint(rgba.x * 255);
|
||||
}
|
||||
|
||||
__global__ void d_render(uint *d_output,
|
||||
uint imageW,
|
||||
uint imageH,
|
||||
float density,
|
||||
float brightness,
|
||||
float transferOffset,
|
||||
float transferScale,
|
||||
cudaTextureObject_t tex,
|
||||
cudaTextureObject_t transferTex)
|
||||
{
|
||||
const int maxSteps = 500;
|
||||
const float tstep = 0.01f;
|
||||
const float opacityThreshold = 0.95f;
|
||||
const float3 boxMin = make_float3(-1.0f, -1.0f, -1.0f);
|
||||
const float3 boxMax = make_float3(1.0f, 1.0f, 1.0f);
|
||||
|
||||
uint x = blockIdx.x * blockDim.x + threadIdx.x;
|
||||
uint y = blockIdx.y * blockDim.y + threadIdx.y;
|
||||
|
||||
if ((x >= imageW) || (y >= imageH))
|
||||
return;
|
||||
|
||||
float u = (x / (float)imageW) * 2.0f - 1.0f;
|
||||
float v = (y / (float)imageH) * 2.0f - 1.0f;
|
||||
|
||||
// calculate eye ray in world space
|
||||
Ray eyeRay;
|
||||
eyeRay.o = make_float3(mul(c_invViewMatrix, make_float4(0.0f, 0.0f, 0.0f, 1.0f)));
|
||||
eyeRay.d = normalize(make_float3(u, v, -2.0f));
|
||||
eyeRay.d = mul(c_invViewMatrix, eyeRay.d);
|
||||
|
||||
// find intersection with box
|
||||
float tnear, tfar;
|
||||
int hit = intersectBox(eyeRay, boxMin, boxMax, &tnear, &tfar);
|
||||
|
||||
if (!hit)
|
||||
return;
|
||||
|
||||
if (tnear < 0.0f)
|
||||
tnear = 0.0f; // clamp to near plane
|
||||
|
||||
// march along ray from front to back, accumulating color
|
||||
float4 sum = make_float4(0.0f);
|
||||
float t = tnear;
|
||||
float3 pos = eyeRay.o + eyeRay.d * tnear;
|
||||
float3 step = eyeRay.d * tstep;
|
||||
|
||||
for (int i = 0; i < maxSteps; i++) {
|
||||
// read from 3D texture
|
||||
// remap position to [0, 1] coordinates
|
||||
float sample = tex3D<float>(tex, pos.x * 0.5f + 0.5f, pos.y * 0.5f + 0.5f, pos.z * 0.5f + 0.5f);
|
||||
// sample *= 64.0f; // scale for 10-bit data
|
||||
|
||||
// lookup in transfer function texture
|
||||
float4 col = tex1D<float4>(transferTex, (sample - transferOffset) * transferScale);
|
||||
col.w *= density;
|
||||
|
||||
// "under" operator for back-to-front blending
|
||||
// sum = lerp(sum, col, col.w);
|
||||
|
||||
// pre-multiply alpha
|
||||
col.x *= col.w;
|
||||
col.y *= col.w;
|
||||
col.z *= col.w;
|
||||
// "over" operator for front-to-back blending
|
||||
sum = sum + col * (1.0f - sum.w);
|
||||
|
||||
// exit early if opaque
|
||||
if (sum.w > opacityThreshold)
|
||||
break;
|
||||
|
||||
t += tstep;
|
||||
|
||||
if (t > tfar)
|
||||
break;
|
||||
|
||||
pos += step;
|
||||
}
|
||||
|
||||
sum *= brightness;
|
||||
|
||||
// write output color
|
||||
d_output[y * imageW + x] = rgbaFloatToInt(sum);
|
||||
}
|
||||
|
||||
extern "C" void setTextureFilterMode(bool bLinearFilter)
|
||||
{
|
||||
if (texObject) {
|
||||
checkCudaErrors(cudaDestroyTextureObject(texObject));
|
||||
}
|
||||
cudaResourceDesc texRes;
|
||||
memset(&texRes, 0, sizeof(cudaResourceDesc));
|
||||
|
||||
texRes.resType = cudaResourceTypeArray;
|
||||
texRes.res.array.array = d_volumeArray;
|
||||
|
||||
cudaTextureDesc texDescr;
|
||||
memset(&texDescr, 0, sizeof(cudaTextureDesc));
|
||||
|
||||
texDescr.normalizedCoords = true;
|
||||
texDescr.filterMode = bLinearFilter ? cudaFilterModeLinear : cudaFilterModePoint;
|
||||
|
||||
texDescr.addressMode[0] = cudaAddressModeWrap;
|
||||
texDescr.addressMode[1] = cudaAddressModeWrap;
|
||||
texDescr.addressMode[2] = cudaAddressModeWrap;
|
||||
|
||||
texDescr.readMode = cudaReadModeNormalizedFloat;
|
||||
|
||||
checkCudaErrors(cudaCreateTextureObject(&texObject, &texRes, &texDescr, NULL));
|
||||
}
|
||||
|
||||
extern "C" void initCuda(void *h_volume, cudaExtent volumeSize)
|
||||
{
|
||||
// create 3D array
|
||||
cudaChannelFormatDesc channelDesc = cudaCreateChannelDesc<VolumeType>();
|
||||
checkCudaErrors(cudaMalloc3DArray(&d_volumeArray, &channelDesc, volumeSize));
|
||||
|
||||
// copy data to 3D array
|
||||
cudaMemcpy3DParms copyParams = {0};
|
||||
copyParams.srcPtr =
|
||||
make_cudaPitchedPtr(h_volume, volumeSize.width * sizeof(VolumeType), volumeSize.width, volumeSize.height);
|
||||
copyParams.dstArray = d_volumeArray;
|
||||
copyParams.extent = volumeSize;
|
||||
copyParams.kind = cudaMemcpyHostToDevice;
|
||||
checkCudaErrors(cudaMemcpy3D(©Params));
|
||||
|
||||
cudaResourceDesc texRes;
|
||||
memset(&texRes, 0, sizeof(cudaResourceDesc));
|
||||
|
||||
texRes.resType = cudaResourceTypeArray;
|
||||
texRes.res.array.array = d_volumeArray;
|
||||
|
||||
cudaTextureDesc texDescr;
|
||||
memset(&texDescr, 0, sizeof(cudaTextureDesc));
|
||||
|
||||
texDescr.normalizedCoords = true; // access with normalized texture coordinates
|
||||
texDescr.filterMode = cudaFilterModeLinear; // linear interpolation
|
||||
|
||||
texDescr.addressMode[0] = cudaAddressModeClamp; // clamp texture coordinates
|
||||
texDescr.addressMode[1] = cudaAddressModeClamp;
|
||||
texDescr.addressMode[2] = cudaAddressModeClamp;
|
||||
|
||||
texDescr.readMode = cudaReadModeNormalizedFloat;
|
||||
|
||||
checkCudaErrors(cudaCreateTextureObject(&texObject, &texRes, &texDescr, NULL));
|
||||
|
||||
// create transfer function texture
|
||||
float4 transferFunc[] = {
|
||||
{
|
||||
0.0,
|
||||
0.0,
|
||||
0.0,
|
||||
0.0,
|
||||
},
|
||||
{
|
||||
1.0,
|
||||
0.0,
|
||||
0.0,
|
||||
1.0,
|
||||
},
|
||||
{
|
||||
1.0,
|
||||
0.5,
|
||||
0.0,
|
||||
1.0,
|
||||
},
|
||||
{
|
||||
1.0,
|
||||
1.0,
|
||||
0.0,
|
||||
1.0,
|
||||
},
|
||||
{
|
||||
0.0,
|
||||
1.0,
|
||||
0.0,
|
||||
1.0,
|
||||
},
|
||||
{
|
||||
0.0,
|
||||
1.0,
|
||||
1.0,
|
||||
1.0,
|
||||
},
|
||||
{
|
||||
0.0,
|
||||
0.0,
|
||||
1.0,
|
||||
1.0,
|
||||
},
|
||||
{
|
||||
1.0,
|
||||
0.0,
|
||||
1.0,
|
||||
1.0,
|
||||
},
|
||||
{
|
||||
0.0,
|
||||
0.0,
|
||||
0.0,
|
||||
0.0,
|
||||
},
|
||||
};
|
||||
|
||||
cudaChannelFormatDesc channelDesc2 = cudaCreateChannelDesc<float4>();
|
||||
cudaArray *d_transferFuncArray;
|
||||
checkCudaErrors(cudaMallocArray(&d_transferFuncArray, &channelDesc2, sizeof(transferFunc) / sizeof(float4), 1));
|
||||
checkCudaErrors(cudaMemcpy2DToArray(
|
||||
d_transferFuncArray, 0, 0, transferFunc, 0, sizeof(transferFunc), 1, cudaMemcpyHostToDevice));
|
||||
|
||||
memset(&texRes, 0, sizeof(cudaResourceDesc));
|
||||
|
||||
texRes.resType = cudaResourceTypeArray;
|
||||
texRes.res.array.array = d_transferFuncArray;
|
||||
|
||||
memset(&texDescr, 0, sizeof(cudaTextureDesc));
|
||||
|
||||
texDescr.normalizedCoords = true; // access with normalized texture coordinates
|
||||
texDescr.filterMode = cudaFilterModeLinear;
|
||||
|
||||
texDescr.addressMode[0] = cudaAddressModeClamp; // wrap texture coordinates
|
||||
|
||||
texDescr.readMode = cudaReadModeElementType;
|
||||
|
||||
checkCudaErrors(cudaCreateTextureObject(&transferTex, &texRes, &texDescr, NULL));
|
||||
}
|
||||
|
||||
extern "C" void freeCudaBuffers()
|
||||
{
|
||||
checkCudaErrors(cudaDestroyTextureObject(texObject));
|
||||
checkCudaErrors(cudaDestroyTextureObject(transferTex));
|
||||
checkCudaErrors(cudaFreeArray(d_volumeArray));
|
||||
checkCudaErrors(cudaFreeArray(d_transferFuncArray));
|
||||
}
|
||||
|
||||
extern "C" void render_kernel(dim3 gridSize,
|
||||
dim3 blockSize,
|
||||
uint *d_output,
|
||||
uint imageW,
|
||||
uint imageH,
|
||||
float density,
|
||||
float brightness,
|
||||
float transferOffset,
|
||||
float transferScale)
|
||||
{
|
||||
d_render<<<gridSize, blockSize>>>(
|
||||
d_output, imageW, imageH, density, brightness, transferOffset, transferScale, texObject, transferTex);
|
||||
}
|
||||
|
||||
extern "C" void copyInvViewMatrix(float *invViewMatrix, size_t sizeofMatrix)
|
||||
{
|
||||
checkCudaErrors(cudaMemcpyToSymbol(c_invViewMatrix, invViewMatrix, sizeofMatrix));
|
||||
}
|
||||
|
||||
#endif // #ifndef _VOLUMERENDER_KERNEL_CU_
|
||||
Reference in New Issue
Block a user